﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Win32;

using Pixoneer.NXDL;       // 기존 함수 관련 기능
using Pixoneer.NXDL.NCC;   // 좌표 시스템 관련기능
using Pixoneer.NXDL.NIO;   // 파일 입출력 관련기능
using Pixoneer.NXDL.NRS;   // Remote Secsing 관련기능
using Pixoneer.NXDL.NGR;   // Graphic 관련 기능
using Pixoneer.NXDL.NXMap; // MapView 관련 기능

namespace XDL_MapView1
{
    /// <summary>
    /// MainWindow.xaml에 대한 상호 작용 논리
    /// </summary>
    public partial class MainWindow : Window
    {
        public XRasterIO m_RasterIO; // 영상의 입출력을 담당할 객체 선언
        
        public MainWindow()
        {
            InitializeComponent();

            String StrError;
            m_RasterIO = new XRasterIO();  // 객체 생성
            if(m_RasterIO.Initialize(out StrError) == false)  // 영상 입출력 객체 초기화
            {
                MessageBox.Show(StrError);
            }

            nxMapLayerComposites1.LayerVisible = true; // 영상레이어를 보이도록 속성을 변경
        }

        private void openFileMenuItem_Click(object sender, RoutedEventArgs e)
        {
            // 1. 파일 Open을 수정한다.
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Filter = "XDM file(*.xdm)|*.XDM||";
            openFileDialog.RestoreDirectory = true;

            Nullable<bool> result = openFileDialog.ShowDialog();       
            if (result != true) return;

            string strFilePath = openFileDialog.FileName;

            // 2. FileDriverManger를 통해 XRSLoadFile을 생성한다.
            string strError;
            XRSLoadFile xrsFileInput = m_RasterIO.LoadFile(strFilePath, out strError, false, eIOCreateXLDMode.All_NoMsg);
            if (xrsFileInput == null) return;

            // 3. 기존에 생성되어 있는 Composite가 있다면 Lock을 걸어 도시 수행을 잠시 멈춘다.
            nxMapLayerComposites1.Lock();

            // 4. nxMapLayerComposites1개체에 존재하는 XDMCompManager를 꺼낸다. XDMCompManager가 실제로 Composite들의 관리를 수행한다.
            XDMCompManager xdmCompManager = nxMapLayerComposites1.GetXDMCompManager();

            // 5. 기존에 Composite가 존재한다면 모두 삭제한다.
            for (int i = 0; i < xdmCompManager.NumComp; i++)
            {
                XDMComposite comp = xdmCompManager.GetXDMCompositeAt(i);
                comp.Dispose();
            }

            xdmCompManager.RemoveXDMCompositeAll();

            // 6. XDMComposite를 생성한다.
            XDMComposite newComp = new XDMComposite();

            // 7. 로딩된 파일로부터 밴드의 개수를 칼라로 그릴 것인지 흑백으로 그릴 것인지를 판단한다.
            // 만약 3개 미만인 경우는 Gray모드로 그리고, 3개 이상인 경우는 칼라로 그린다.
            int nNumBand = xrsFileInput.NumBand;
            if (nNumBand < 3)
            {
                // Default로 0번째 밴드를 Gray모드로 그린다. 
                XDMBand band = xrsFileInput.GetBandAt(0);

                // 밴드 이름을 그대로 Composite이름으로 설정
                newComp.Name = band.BandName;

                // Gray Mode로 그리는것을 설정
                newComp.Mode = eCompMode.Gray;

                // Composite의 0번에 Band설정, Gray모드에서는 0번 밴드만을 인식한다. 
                 newComp.SetBand(ref band, 0);
                // 영상 Enhancement를 위해 Histogram의 범위 설정(정규분포 95% 영역)
                newComp.SetCutType(eCompCutType.Ct95, 0);
                // 영상 Enhancement를 위해 Histogram적용시 전체 영역에 대한 Histogram(Band)설정, Visible은 현재 영역
                newComp.SetStretchCoverage(eCompStretchCoverage.Band, 0);
                // 영상 Enhancement를 위해 Histogram의 Stretch형태 설정(Gaussian으로 설정)
                newComp.SetStretchType(eCompStretchType.Gaussian, 0);
            }

            else // 3개 이상의 밴드를 가지고 있는 경우
            {
                XDMBand band1 = xrsFileInput.GetBandAt(0);
                XDMBand band2 = xrsFileInput.GetBandAt(1);
                XDMBand band3 = xrsFileInput.GetBandAt(2);
                newComp.Name = xrsFileInput.FileName; // 파일 이름을 Composite 이름으로 설정

                newComp.Mode = eCompMode.RGB;  // RGB Mode로 그리는 것을 설정
                newComp.SetBand(ref band3, 0); // Composite의 0번에 Band설정, Blue Channel 설정
                newComp.SetBand(ref band2, 1);// Composite의 1번에 Band설정, Green Channel 설정
                newComp.SetBand(ref band1, 2); // Composite의 2번에 Band설정, Red Channel 설정

                for (int i = 0; i < 3; i++)
                {
                    // 영상 Enhancement를 위해 Histogram의 범위 설정(정규분포 95% 영역)
                    newComp.SetCutType(eCompCutType.Ct95, i);
                    // 영상 Enhancement를 위해 Histogram적용 시 전체 영역에 대한 Histogram(Band)설정, Visible은 현재 영역
                    newComp.SetStretchCoverage(eCompStretchCoverage.Band, i);
                    // 영상 Enhancemnet를 위해 Histogram의 Stretch형태 설정(Gaussian으로 설정                        
                    newComp.SetStretchType(eCompStretchType.Gaussian, i);
                }
            }
                
                // 8. 생성된 Composite를 XDMCompManager객체에 추가한다.
                xdmCompManager.AddXDMComposite(ref newComp);

                // 9. 전체 화면 보기를 설정한다.
                nxMapLayerComposites1.ZoomFit();

                // 10. 화면 업데이트를 수행한다.
                nxMapLayerComposites1.Invalidate();

                // 11. 설정된 Lock를 해제한다.
                nxMapLayerComposites1.UnLock();
        }

        private void Window_Closed(object sender, EventArgs e)
        {
            Xfn.Close();
        }
    }
}
